@@ -21,13 +21,15 @@ class VectorIndexDriver(BaseIndexDriver):
     """
 
     def _apply_all(self, docs: 'DocumentSet', *args, **kwargs) -> None:
-        embed_vecs, docs_pts, bad_docs = docs.all_embeddings
+        if self._method_name == 'delete':
+            self.exec_fn(np.array([int(doc.id) for doc in docs]), None)
+        else:
+            embed_vecs, docs_pts, bad_docs = docs.all_embeddings
+            if bad_docs:
+                self.runtime.logger.warning(f'these bad docs can not be added: {bad_docs}')
+            if docs_pts:
+                self.exec_fn(np.array([int(doc.id) for doc in docs_pts]), np.stack(embed_vecs))
 
-        if bad_docs:
-            self.pea.logger.warning(f'these bad docs can not be added: {bad_docs}')
-
-        if docs_pts:
-            self.exec_fn(np.array([int(doc.id) for doc in docs_pts]), np.stack(embed_vecs))
 
 
 class KVIndexDriver(BaseIndexDriver):
