@@ -5,32 +5,33 @@ import os
 from typing import Optional
 
 import numpy as np
+
 from jina.executors.decorators import batching, as_ndarray
 from jina.executors.devices import TorchDevice
 from jina.executors.encoders import BaseEncoder
-from jina.helper import cached_property
-from jina.logging import default_logger
+
 
 def no_grad():
     import torch
     return torch.no_grad()
 
+
 class TransformerTorchEncoder(TorchDevice, BaseEncoder):
     """
     Internally, TransformerTorchEncoder wraps the pytorch version of transformers from huggingface.
     """
 
     def __init__(
-        self,
-        pretrained_model_name_or_path: str = 'sentence-transformers/distilbert-base-nli-stsb-mean-tokens',
-        base_tokenizer_model: Optional[str] = None,
-        pooling_strategy: str = 'mean',
-        layer_index: int = -1,
-        max_length: Optional[int] = None,
-        acceleration: Optional[str] = None,
-        model_save_path: Optional[str] = None,
-        *args,
-        **kwargs,
+            self,
+            pretrained_model_name_or_path: str = 'sentence-transformers/distilbert-base-nli-stsb-mean-tokens',
+            base_tokenizer_model: Optional[str] = None,
+            pooling_strategy: str = 'mean',
+            layer_index: int = -1,
+            max_length: Optional[int] = None,
+            acceleration: Optional[str] = None,
+            model_save_path: Optional[str] = None,
+            *args,
+            **kwargs,
     ):
         """
         :param pretrained_model_name_or_path: Either:
@@ -104,28 +105,20 @@ class TransformerTorchEncoder(TorchDevice, BaseEncoder):
         """Get the file path of the encoder model storage"""
         return self.get_file_from_workspace(self.model_save_path)
 
-    @cached_property
-    def model(self):
+    def post_init(self):
         import torch
-        from transformers import AutoModel
+        from transformers import AutoModel, AutoTokenizer
 
-        model = AutoModel.from_pretrained(
+        self.tokenizer = AutoTokenizer.from_pretrained(self.base_tokenizer_model)
+        self.model = AutoModel.from_pretrained(
             self.pretrained_model_name_or_path, output_hidden_states=True
         )
-        self.to_device(model)
-        
+        self.to_device(self.model)
+
         if self.acceleration == 'quant' and not self.on_gpu:
-            model = torch.quantization.quantize_dynamic(
-                model, {torch.nn.Linear}, dtype=torch.qint8
-            )     
-
-        return model
-
-    @cached_property
-    def tokenizer(self):
-        from transformers import AutoTokenizer
-        tokenizer = AutoTokenizer.from_pretrained(self.base_tokenizer_model)
-        return tokenizer
+            self.model = torch.quantization.quantize_dynamic(
+                self.model, {torch.nn.Linear}, dtype=torch.qint8
+            )
 
     def amp_accelerate(self):
         import torch
@@ -167,7 +160,7 @@ class TransformerTorchEncoder(TorchDevice, BaseEncoder):
             self.logger.error(
                 f'Invalid value {self.layer_index} for `layer_index`,'
                 f' for the model {self.pretrained_model_name_or_path}'
-                f' valid values are integers from {-n_layers} to {n_layers-1}.'
+                f' valid values are integers from {-n_layers} to {n_layers - 1}.'
             )
             raise ValueError
 
@@ -190,7 +183,7 @@ class TransformerTorchEncoder(TorchDevice, BaseEncoder):
             CLS = self.tokenizer.cls_token_id
             ind = torch.nonzero(input_tokens['input_ids'] == CLS)[:, 1]
             ind = ind.unsqueeze(-1).unsqueeze(-1).expand(-1, -1, layer.shape[2])
-            embeddings = torch.gather(layer, 1, ind).squeeze()
+            embeddings = torch.gather(layer, 1, ind).squeeze(dim=1)
         elif self.pooling_strategy == 'mean':
             embeddings = layer.sum(dim=1) / attn_mask.sum(dim=1)
         elif self.pooling_strategy == 'max':
