@@ -27,5 +27,5 @@ class LevenshteinRanker(Match2DocRanker):
         ]
         return np.array(
             new_scores,
-            dtype=[(self.COL_MATCH_HASH, np.int64), (self.COL_SCORE, np.float64)],
+            dtype=[(self.COL_MATCH_ID, np.int64), (self.COL_SCORE, np.float64)],
         )
