@@ -12,6 +12,7 @@ from .uid import *
 from ..ndarray.generic import NdArray
 from ..sets.chunk import ChunkSet
 from ..sets.match import MatchSet
+from ..score import NamedScore
 from ...excepts import BadDocType
 from ...helper import is_url, typename
 from ...importer import ImportExtensions
@@ -40,8 +41,10 @@ class Document:
             d = Document()
             d.text = 'abc'
 
-    Jina requires each Document to have a string id. You can use :meth:`update_id` or simply
-    use :class:`Document` as a context manager:
+    Jina requires each Document to have a string id. You can set a custom one,
+    or if non has been set a random one will be assigned.
+
+    Or you can use :class:`Document` as a context manager:
 
         .. highlight:: python
         .. code-block:: python
@@ -135,6 +138,10 @@ class Document:
                              f'if you are trying to set the content '
                              f'you may use "Document(content=your_content)"') from ex
 
+        if self._document.id is None or not self._document.id:
+            import random
+            self.id = random.randint(0, np.iinfo(np.int64).max)
+
         self.set_attrs(**kwargs)
 
     def __getattr__(self, name: str):
@@ -175,6 +182,14 @@ class Document:
         """Set the modality of the document"""
         self._document.modality = value
 
+    @property
+    def content_hash(self):
+        return self._document.content_hash
+
+    def update_content_hash(self):
+        """Update the document hash according to its content."""
+        self._document.content_hash = get_content_hash(self._document)
+
     @property
     def id(self) -> 'UniqueId':
         """The document id in hex string, for non-binary environment such as HTTP, CLI, HTML and also human-readable.
@@ -189,21 +204,6 @@ class Document:
         """
         return UniqueId(self._document.parent_id)
 
-    def update_id(self):
-        """Update the document id according to its content.
-        .. warning::
-            To fully consider the content in this document, please use this function after
-            you have fully modified the Document, not right way after create the Document.
-            If you are using Document as context manager, then no need to call this function manually.
-            Simply
-            .. highlight:: python
-            .. code-block:: python
-                with Document() as d:
-                    d.text = 'hello'
-                assert d.id  # now `id` has value
-        """
-        self._document.id = new_doc_id(self._document)
-
     @id.setter
     def id(self, value: Union[bytes, str, int]):
         """Set document id to a string value
@@ -402,7 +402,7 @@ class Document:
         return self
 
     def __exit__(self, exc_type, exc_val, exc_tb):
-        self.update_id()
+        self.update_content_hash()
 
     @property
     def content_type(self) -> str:
@@ -453,6 +453,19 @@ class Document:
     def granularity(self, granularity_value: int):
         self._document.granularity = granularity_value
 
+    @property
+    def score(self):
+        return self._document.score
+
+    @score.setter
+    def score(self, value: Union[jina_pb2.NamedScoreProto, NamedScore]):
+        if isinstance(value, jina_pb2.NamedScoreProto):
+            self._document.score.CopyFrom(value)
+        elif isinstance(value, NamedScore):
+            self._document.score.CopyFrom(value._score)
+        else:
+            raise TypeError(f'score is in unsupported type {typename(value)}')
+
     def convert_buffer_to_blob(self, **kwargs):
         """Assuming the :attr:`buffer` is a _valid_ buffer of Numpy ndarray,
         set :attr:`blob` accordingly.
